@@ -57,6 +57,8 @@ class CGSmartImage extends CGExtensions
     parent::SetParameters();
 
     // operational params (what to do, with what...)
+    $this->CreateParameter('alias','',$this->Lang('param_alias'));
+
     $this->SetParameterType('noautoscale',CLEAN_INT);
     $this->CreateParameter('noautoscale','',$this->Lang('param_noautoscale'));
 
@@ -69,6 +71,9 @@ class CGSmartImage extends CGExtensions
     $this->SetParameterType('noauto',CLEAN_INT);
     $this->CreateParameter('noauto','',$this->Lang('param_noauto'));
 
+    $this->SetParameterType('norotate',CLEAN_INT);
+    $this->CreateParameter('norotate','',$this->Lang('param_norotate'));
+
     $this->SetParameterType('notag',CLEAN_INT);
     $this->CreateParameter('notag','',$this->Lang('param_notag'));
 
@@ -104,7 +109,9 @@ class CGSmartImage extends CGExtensions
     $this->SetParameterType('quality',CLEAN_INT);
     $this->CreateParameter('quality','75',$this->Lang('param_quality'));
 
-    $this->SetParameterType(CLEAN_REGEXP.'/filter_.*/',CLEAN_STRING);
+    $this->SetParameterType(CLEAN_REGEXP.'/src\d*/',CLEAN_STRING);
+    $this->SetParameterType(CLEAN_REGEXP.'/alias\d*/',CLEAN_STRING);
+    $this->SetParameterType(CLEAN_REGEXP.'/filter_\w*/',CLEAN_STRING);
     $this->CreateParameter('filter_blur','',$this->Lang('param_filter_blur'));
     $this->CreateParameter('filter_brightness','',$this->Lang('param_filter_brightness'));
     $this->CreateParameter('filter_colorize','',$this->Lang('param_filter_colorize'));
@@ -113,6 +120,7 @@ class CGSmartImage extends CGExtensions
     $this->CreateParameter('filter_croptofit','',$this->Lang('param_filter_croptofit'));
     $this->CreateParameter('filter_edgedetect','',$this->Lang('param_filter_edgedetect'));
     $this->CreateParameter('filter_emboss','',$this->Lang('param_filter_emboss'));
+    $this->CreateParameter('filter_flip','',$this->Lang('param_filter_flip'));
     $this->CreateParameter('filter_grayscale','',$this->Lang('param_filter_grayscale'));
     $this->CreateParameter('filter_meanremoval','',$this->Lang('param_filter_meanremoval'));
     $this->CreateParameter('filter_negate','',$this->Lang('param_filter_negate'));
@@ -148,7 +156,7 @@ class CGSmartImage extends CGExtensions
    ---------------------------------------------------------*/
   function GetVersion()
   {
-    return '1.2.1';
+    return '1.3';
   }
 
   
