@@ -59,12 +59,145 @@ $opp['overwrite'] = 0;
 $opp['nocache'] = 0;
 $opp['noembed'] = 0;
 $opp['noauto'] = 0;
+$opp['norotate'] = 0;
 $opp['noautoscale'] = 0;
 $opp['notag'] = 0;
 $opp['src'] = '';
 $opp['quality'] = 75;
 $opp['filters'] = array();
 
+$tmp = $this->GetPreference('aliases');
+if( $tmp ) $aliases = unserialize($tmp);
+
+if( !function_exists('cgsi_get_alias_options') )
+  {
+    function cgsi_get_alias_options($name)
+    {
+      static $aliases;
+      if( !$aliases )
+	{
+	  $mod = cms_utils::get_module('CGSmartImage');
+	  $tmp = $mod->GetPreference('aliases');
+	  if( !$tmp ) 
+	    $aliases = 'FALSE';
+	  else
+	    $aliases = unserialize($tmp);
+	}
+      if( is_array($aliases) )
+	{
+	  $keys = array_keys($aliases);
+	  for( $i = 0; $i < count($keys); $i++ )
+	    {
+	      $key = $keys[$i];
+	      if( $aliases[$key]['name'] == $name )
+		{
+		  return $aliases[$key]['options'];
+		}
+	    }
+	}
+    }
+
+    function cgsi_expand_quoted_string($str)
+    {
+      $result = array();
+      $col = '';
+      $safe = '';
+      $prev_char = '';
+      for( $i = 0; $i < strlen($str); $i++ )
+	{
+	  switch( $str[$i] )
+	    {
+	    case ' ':
+	      if( !$safe )
+		{
+		  if( strpos($col,'=') !== FALSE )
+		    {
+		      list($k,$v) = explode('=',$col,2);
+		      $result[$k] = $v;
+		    }
+		  $col = '';
+		}
+	      break;
+
+	    case "'":
+	    case '"':
+	      if( $prev_char != '\\' )
+		{
+		  if( $str[$i] == $safe )
+		    {
+		      $safe = null;
+		    }
+		  else
+		    {
+		      $safe = $str[$i];
+		    }
+		}
+	      break;
+
+	    default:
+	      $col .= $str[$i];
+	      break;
+	    }
+
+	  $prev_char = $str[$i];
+	}
+
+      if( strlen($col) != 0 )
+	{
+	  if( strpos($col,'=') !== FALSE )
+	    {
+	      list($k,$v) = explode('=',$col,2);
+	      $result[$k] = $v;
+	    }
+	}
+
+      return $result;
+    }
+} // function exists
+
+// first pass... expand aliases and build src
+$new_params = array();
+foreach( $params as $key => $value )
+{
+  if( startswith($key,'alias') )
+    {
+      // expand alias
+      $options = cgsi_get_alias_options($value);
+      if( $options )
+	{
+	  // parse a string into an array of arguments.
+	  $data = cgsi_expand_quoted_string($options);
+	  if( is_array($data) )
+	    {
+	      foreach( $data as $key => $value )
+		{
+		  $new_params[$key] = $value;
+		}
+	    }
+	  continue;
+	}
+    }
+  elseif( startswith($key,'src') && $key != 'src' )
+    {
+      // handle src1 through src99 arguments.
+      if( !isset($new_params['src']) )
+	{
+	  $new_params['src'] = '';
+	}
+      if( !empty($new_params['src']) && !endswith($new_params['src'],'/') )
+	{
+	  $new_params['src'] .= '/';
+	}
+      $new_params['src'] .= $value;
+      continue;
+    }
+
+  // everything else just gets added.
+  $new_params[$key] = $value;
+}
+$params = $new_params;
+
+// second pass, built our arrays
 $error = '';
 foreach( $params as $key => $value )
 {
@@ -82,7 +215,6 @@ foreach( $params as $key => $value )
 
       // add it to the ops.
       $opp['filters'][] = array($classname,$args);
-
       // done.
       continue;
     }
@@ -94,15 +226,15 @@ foreach( $params as $key => $value )
     case 'name':
     case 'alt':
     case 'style':
-      $outp[$key] = trim($params[$key]);
+      $outp[$key] = trim($value);
       break;
     case 'width':
     case 'height':
-      $outp[$key] = (int)$params[$key];
+      $outp[$key] = (int)$value;
       break;
 
     case 'src':
-      $opp[$key] = trim($params[$key]);
+      $opp[$key] = trim($value);
       break;
 
     case 'quality':
@@ -114,6 +246,7 @@ foreach( $params as $key => $value )
     case 'notag':
     case 'nocache':
     case 'noembed':
+    case 'norotate':
       $opp[$key] = cge_utils::to_bool($value);
       break;
     }
@@ -123,42 +256,107 @@ if( !$error && !$opp['src'] )
     $error = $this->Lang('error_missingparam','src');
   }
 
-
 //
 // find the source image ... the actual filename
 //
-if( file_exists($opp['src']) )
+if( !$error && file_exists($opp['src']) )
   {
     // user specified the complete path to the file.
     $srcfile = $opp['src'];
     if( startswith($opp['src'],'/') ) $rel = 1;
   }    
-if( !$srcfile && startswith($opp['src'],$config['uploads_url']) )
+if( !$error && !$srcfile && startswith($opp['src'],$config['uploads_url']) )
   {
     $tmp = str_replace($config['uploads_url'],$config['uploads_path'],$opp['src']);
     if( file_exists($tmp) ) $srcfile = $tmp;
   }
-if( !$srcfile && startswith($opp['src'],$config['root_url']) )
+if( !$error && !$srcfile && startswith($opp['src'],$config['root_url']) )
   {
     $tmp = str_replace($config['root_url'],$config['root_path'],$opp['src']);
     if( file_exists($tmp) ) $srcfile = $tmp;
   }
-if( !$srcfile && startswith($opp['src'],'/') )
+if( !$error && !$srcfile && startswith($opp['src'],'/') )
   {
     $tmp = $config['root_path'].$opp['src'];
     if( file_exists($tmp) ) $srcfile = $tmp;
   }
-if( !$srcfile && isset($config['ssl_url']) && startswith($opp['src'],$config['ssl_url']) )
+if( !$error && !$srcfile && isset($config['ssl_url']) && startswith($opp['src'],$config['ssl_url']) )
   {
     $tmp = str_replace($config['ssl_url'],$config['root_path'],$opp['src']);
     if( file_exists($tmp) ) $srcfile = $tmp;
   }
-
-if( !$srcfile )
+// if( !$error && !$srcfile && (startswith($opp['src'],'http:') || startswith($opp['src'],'https:') || startswith($opp['src'],'ftp:')) )
+//   {
+//     // okay, gotta assume that ths is a remote file
+//     // get it, and cache it.
+//   }
+if( !$error && !$srcfile )
   {
     $error = $this->Lang('error_srcnotfound',$opp['src']);
   }
 
+// are we automagically rotating.
+if( !$opp['norotate'] && function_exists('exif_read_data') )
+  {
+    // if there is already a rotate filter in the list, we won't override that.
+    $fn = 0;
+    for( $f = 0; $f < count($opp['filters']); $f++ )
+      {
+	if( $opp['filters'][$f][0] == 'CGImage_Rotate_Filter' )
+	  {
+	    $fn = 1;
+	    break;
+	  }
+      }
+
+    if( $fn == 0 )
+      {
+	// we can try to read the exif information to find an orientation.
+	$exif = exif_read_data($srcfile,0,TRUE);
+	if( is_array($exif) && isset($exif['IFD0']) && isset($exif['IFD0']['Orientation']) &&
+	    is_int($exif['IFD0']['Orientation']) )
+	  {
+
+	    // found an orientation, now we gotta figure out what filters to add.
+	    $orientation = (int)$exif['IFD0']['Orientation'];
+	    $new_filters = array();
+	    switch( $orientation )
+	      {
+	      case 1:
+		// nothing.
+		break;
+	      case 2:
+		// horizontal flip.
+		$new_filters[] = array('CGImage_Flip_Filter',0);
+		break;
+	      case 3:
+		// rotate 180
+		$new_filters[] = array('CGImage_Rotate_Filter',array(180,0));
+		break;
+	      case 4:
+		$new_filters[] = array('CGImage_Flip_Filter',1);
+		break;
+	      case 5:
+		$new_filters[] = array('CGImage_Flip_Filter',1);
+		$new_filters[] = array('CGImage_Rotate_Filter',array(90,0));
+		break;
+	      case 6:
+		$new_filters[] = array('CGImage_Rotate_Filter',array(90,0));
+		break;
+	      case 7:
+		$new_filters[] = array('CGImage_Flip_Filter',0);
+		$new_filters[] = array('CGImage_Rotate_Filter',array(90,0));
+		break;
+	      case 8:
+		$new_filters[] = array('CGImage_Rotate_Filter',array(-90,0));
+		break;
+	      }
+
+	    $opp['filters'] = array_merge($new_filters,$opp['filters']);
+	  }
+      }
+  }
+
 // now do some intelligence, and see if the width, and height parameters passed in
 // match the image size... if not, add a filter.
 if( !$error && ($outp['width'] > 0 || $outp['height'] > 0) && $opp['noautoscale'] == 0 )
@@ -187,7 +385,7 @@ if( !$error && ($outp['width'] > 0 || $outp['height'] > 0) && $opp['noautoscale'
 	    {
 	      // gonna do a resize
 	      $filter = 'CGImage_Resize_Filter';
-	      $args = array('w',(int)$outp['width']);
+	      $args = array('h',(int)$outp['height']);
 	    }
 	  if( is_array($args) && $filter != '' )
 	    {
@@ -209,6 +407,10 @@ if( !$error && ($outp['width'] > 0 || $outp['height'] > 0) && $opp['noautoscale'
     }
 }
 
+//
+// end of smartness stuff... now begin the work
+//
+
 if( !$error && count(array_keys($opp['filters'])) )
   {
     //
@@ -235,6 +437,7 @@ if( !$error && $srcfile && $want_transform )
       {
 	// see if it exists
 	$dest_fname = $destdir.'/'.$destname;
+
 	$dest_url = $this->get_cached_image_url($destname);
 	if( !file_exists($dest_fname) || $opp['overwrite']  )
 	  {
